/*

Description
-----------
    GameCron plugin is for delayed command execution counting from every map start.
    
    http://forums.alliedmods.net/showthread.php?t=91619

Commands and Cvars
------------------
    sm_gamecron_version                          - GameCron version.
    
    sm_gamecron_add <seconds> <command>          - Add delayed command.
    sm_gamecron_del <id>                         - Delete task by id.
    sm_gamecron_clear                            - Delete all tasks.
    sm_gamecron_list                             - List all current tasks.
    sm_gamecron_restart                          - Restart all timers for tasks.

Requirements
------------
    Counter-Strike: Source
    SourceMod 1.2.0

Changelog
---------
    1.0.3:
        * CPU usage optimization.
        
    1.0.2:
        + Added sm_gamecron_version.
        
    1.0.1:
        + Added restart hooking. It was commented due to compatibility with SourceMod 1.0.0.
        - Debug log messages removed.

Credits
-------

TODO
----

*/

#include <sourcemod>

#pragma semicolon 1

#define MAX_TASKS 10
#define MAX_TASK_LEN 100

#define PLUGIN_VERSION "1.0.3"

public Plugin:myinfo = 
{
    name = "GameCron",
    author = "Otstrel.ru Team",
    description = "Simple crontab for delayed command execution every map start.",
    version = PLUGIN_VERSION,
    url = "http://otstrel.ru"
};

new Float:g_fTimerDelay[MAX_TASKS];
new String:g_sTimerCommand[MAX_TASKS][MAX_TASK_LEN];
new Handle:g_hTimer[MAX_TASKS];

new Handle:mp_restartgame = INVALID_HANDLE;
    
public OnPluginStart()
{
    new Handle:Cvar_Version = CreateConVar("sm_gamecron_version", PLUGIN_VERSION,    "GameCron Version", FCVAR_SPONLY|FCVAR_REPLICATED|FCVAR_NOTIFY);
    /* Just to make sure they it updates the convar version if they just had the plugin reload on map change */
    SetConVarString(Cvar_Version, PLUGIN_VERSION);
    
    ClearTasks();
    
    RegAdminCmd ("sm_gamecron_add", Command_Add, ADMFLAG_ROOT, "sm_gamecron_add time command - Add delayed command.");
    RegAdminCmd ("sm_gamecron_del", Command_Del, ADMFLAG_ROOT, "sm_gamecron_del id - Delete task by id.");
    RegAdminCmd ("sm_gamecron_clear", Command_Clear, ADMFLAG_ROOT, "sm_gamecron_clear - Delete all tasks.");
    RegAdminCmd ("sm_gamecron_list", Command_List, ADMFLAG_ROOT, "sm_gamecron_list - List all current tasks.");
    RegAdminCmd ("sm_gamecron_restart", Command_Restart, ADMFLAG_ROOT, "sm_gamecron_restart - Restart all timers for tasks.");
    
    mp_restartgame = FindConVar("mp_restartgame");
    if ( mp_restartgame != INVALID_HANDLE )
    {
        HookConVarChange(mp_restartgame, Restart_Handler);
    }
}

public Restart_Handler(Handle:convar, const String:oldValue[], const String:newValue[])
{
    if ( (convar == mp_restartgame) && (GetConVarInt(mp_restartgame) == 0) )
    {
        RestartTimer();
    }
}

public OnPluginEnd()
{
    ClearTasks();
}

public OnMapStart()
{
    RestartTimer();
}

public RestartTimer()
{
    for (new i = 0; i < MAX_TASKS; i++)
    {
        if ( g_hTimer[i] && ( g_hTimer[i] != INVALID_HANDLE) )
        {
            CloseHandle(g_hTimer[i]);
            g_hTimer[i] = INVALID_HANDLE;
        }
        if (g_fTimerDelay[i] > 0)
        {
            g_hTimer[i] = CreateTimer(g_fTimerDelay[i], Timer_ExecuteTask, i);
        }
    }
}

public ClearTasks()
{
    for (new i = 0; i < MAX_TASKS; i++)
    {
        if ( g_hTimer[i] && ( g_hTimer[i] != INVALID_HANDLE) )
        {
            CloseHandle(g_hTimer[i]);
            g_hTimer[i] = INVALID_HANDLE;
        }
        g_sTimerCommand[i] = "";
        g_fTimerDelay[i] = 0.0;
    }
}

public Action:Timer_ExecuteTask(Handle:timer, any:taskId)
{
    g_hTimer[taskId] = INVALID_HANDLE;
    ServerCommand(g_sTimerCommand[taskId]);
    return Plugin_Continue;
}

public Action:Command_Clear(client, args)
{
    ClearTasks();
    ReplyToCommand (client, "[GameCron] All tasks deleted.");
    return Plugin_Handled;
}

public Action:Command_Del(client, args)
{
    decl String:buffer[MAX_TASK_LEN] = "";
    if (args == 1)
    {
        GetCmdArg (1, buffer, sizeof(buffer));
        new iId = StringToInt(buffer);
        if (DelTask(iId))
        {
            ReplyToCommand (client, "[GameCron] Cron task %i deleted successfully.", iId);
        }
        else
        {
            ReplyToCommand (client, "[GameCron] Can not find task.");
        }
    }
    else
    {
        ReplyToCommand (client, "[GameCron] sm_gamecron_del - Invalid usage.");
        ReplyToCommand (client, "[GameCron] Usage: sm_gamecron_del id");
    }
    return Plugin_Handled;
}

public Action:Command_List(client, args)
{
    ReplyToCommand (client, "[GameCron] All tasks list:");
    for (new i = 0; i < MAX_TASKS; i++)
    {
        if (g_fTimerDelay[i] > 0)
        {
            ReplyToCommand (client, "%i\t%f\t%s",i,g_fTimerDelay[i],g_sTimerCommand[i]);
        }
    }
    return Plugin_Handled;
}

public Action:Command_Restart(client, args)
{
    RestartTimer(); 
    ReplyToCommand (client, "[GameCron] Restarted.");
    return Plugin_Handled;
}

public Action:Command_Add(client, args)
{
    decl String:buffer[MAX_TASK_LEN] = "";
    if (args == 2)
    {
        GetCmdArg (1, buffer, sizeof(buffer));
        new Float:fDelay = StringToFloat(buffer);
        GetCmdArg (2, buffer, sizeof(buffer));
        if (strcmp(buffer,"")!=0)
        {
            if (AddTask(fDelay,buffer))
            {
                ReplyToCommand (client, "[GameCron] Cron task \"%s\" added successfully.", buffer);
            }
            else
            {
                ReplyToCommand (client, "[GameCron] Task list is full.");
            }
        }
    }
    else
    {
        ReplyToCommand (client, "[GameCron] sm_gamecron_add - Invalid usage.");
        ReplyToCommand (client, "[GameCron] Usage: sm_gamecron_add time \"command\"");
    }
    return Plugin_Handled;
}

public bool:AddTask(Float:fDelay, const String:sCommand[MAX_TASK_LEN])
{
    for (new i = 0; i < MAX_TASKS; i++)
    {
        if (g_fTimerDelay[i] <= 0)
        {
            g_fTimerDelay[i] = fDelay;
            g_sTimerCommand[i] = sCommand;
            g_hTimer[i] = CreateTimer(g_fTimerDelay[i], Timer_ExecuteTask, i);
            return true;
        }
    }
    return false;
}

public bool:DelTask(iId)
{
    if (iId < 0 || iId > MAX_TASKS)
    {
        return false;
    }
    if ( g_fTimerDelay[iId] <= 0 )
    {
        return false;
    }
    g_fTimerDelay[iId] = 0.0;
    g_sTimerCommand[iId] = "";
    if ( g_hTimer[iId] && ( g_hTimer[iId] != INVALID_HANDLE) )
    {
        CloseHandle(g_hTimer[iId]);
        g_hTimer[iId] = INVALID_HANDLE;
    }
    return true;
}
